package github.benjamin.equipreservebackend.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import github.benjamin.equipreservebackend.dto.UserDTO;
import github.benjamin.equipreservebackend.entity.Role;
import github.benjamin.equipreservebackend.entity.Team;
import github.benjamin.equipreservebackend.entity.User;
import github.benjamin.equipreservebackend.exception.ApiException;
import github.benjamin.equipreservebackend.mapper.RoleMapper;
import github.benjamin.equipreservebackend.mapper.TeamMapper;
import github.benjamin.equipreservebackend.mapper.UserMapper;
import github.benjamin.equipreservebackend.response.ResponseCode;
import github.benjamin.equipreservebackend.security.SecurityUser;
import github.benjamin.equipreservebackend.service.UserService;
import github.benjamin.equipreservebackend.utils.JwtUtil;
import github.benjamin.equipreservebackend.utils.PasswordUtil;
import github.benjamin.equipreservebackend.vo.UserVO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class UserServiceImpl implements UserService {

    private final UserMapper userMapper;

    private final RoleMapper roleMapper;

    private final TeamMapper teamMapper;

    private final JwtUtil jwtUtil;

    @Override
    public User login(String username, String password) {
        User user = userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getUsername, username));
        if (Objects.isNull(user)) {
            throw new ApiException(ResponseCode.USER_NOT_EXIST, "用户不存在");
        }
        if (!PasswordUtil.matches(password, user.getPassword())) {
            throw new ApiException(ResponseCode.PASSWORD_ERROR, "密码错误");
        }
        return user;
    }

    @Override
    public SecurityUser loadSecurityUserById(Long userId) {
        User user = userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getId, userId));
        List<Role> roles = roleMapper.selectRoleByUserId(userId);
        return new SecurityUser(user, roles);
    }

    @Override
    public UserVO getUserVO(Long id) {
        User user = userMapper.selectById(id);
        Team team = teamMapper.selectById(user.getTeamId());
        return new UserVO(user, team);
    }

    @Override
    public void updateUser(Long id, UserDTO dto) {
        LambdaUpdateWrapper<User> wrapper = new LambdaUpdateWrapper<User>()
                .eq(User::getId, id)
                .set(StringUtils.hasText(dto.getName()), User::getName, dto.getName())
                .set(StringUtils.hasText(dto.getPhone()), User::getPhone, dto.getPhone());
        if (StringUtils.hasText(dto.getPassword())) {
            String encoded = PasswordUtil.encode(dto.getPassword());
            wrapper.set(User::getPassword, encoded);
        }
        userMapper.update(wrapper);
    }
}
